<template>
  <LoadingLayout :loading="state.loading">
    <AppContainer>
      <ProTableQueryFilterBar @on-reset="reset">
        <template #query>
          <QueryFilterItem>
            <SearchInput
              v-model="extraParamState.queryCondition"
              style="width: 200px"
              placeholder="账号/姓名/手机号"
              @on-click-search="getList"
            >
            </SearchInput>
          </QueryFilterItem>
        </template>
        <template #btn>
          <el-button @click="openDialog()" icon="Plus" type="primary">新增</el-button>
        </template>
      </ProTableQueryFilterBar>
      <ProTableV2 v-bind="proTableProps" :columns="column" :operationBtns="operationBtns">
      </ProTableV2>
    </AppContainer>
    <AddOrEditAccountDialog v-bind="dialogProps" />
    <ResetPasswordDialog v-bind="resetPasswordDialogProps"></ResetPasswordDialog>
  </LoadingLayout>
</template>

<script setup lang="ts">
import {
  ProTableQueryFilterBar,
  ProTableV2,
  SearchInput,
  LoadingLayout,
  AppContainer,
  QueryFilterItem,
  useTable,
  useFormDialog,
  defineOperationBtns,
} from '@bole-core/components';
import * as userRoleServices from '@/services/api/UserRole';
import * as accountServices from '@/services/api/Account';
import { Message, OrderInputType } from '@bole-core/core';
import AddOrEditAccountDialog from './components/AddOrEditAccountDialog.vue';
import ResetPasswordDialog from './components/ResetPasswordDialog.vue';
import { formatRoleName } from '@/utils';

defineOptions({
  name: 'AccountManageList',
});

const column: API.CustomModuleColumnDto[] = [
  {
    id: '1',
    enCode: 'userName',
    name: '账号',
  },
  {
    id: '2',
    enCode: 'name',
    name: '姓名',
  },
  {
    id: '3',
    enCode: 'phoneNumber',
    name: '手机号',
  },
  {
    id: '4',
    enCode: 'remark',
    name: '备注',
  },
];

const operationBtns = defineOperationBtns([
  {
    data: {
      enCode: 'editBtn',
      name: '编辑',
    },
    emits: {
      onClick: (role) => openDialog(role),
    },
  },
  {
    data: {
      enCode: 'resetPasswordBtn',
      name: '重置密码',
    },
    emits: {
      onClick: (role) => openResetPasswordDialog(role),
    },
  },
]);

const BaseState = {
  loading: true,
};

const state = reactive({ ...BaseState });

onMounted(async () => {
  await getList();
  state.loading = false;
});

const {
  getDataSource: getList,
  proTableProps,
  paginationState,
  extraParamState,
  reset,
} = useTable(
  async ({ pageIndex, pageSize }, extraParamState) => {
    try {
      let params: API.GetBackClientUsersInput = {
        pageModel: {
          rows: pageSize,
          page: pageIndex,
          orderInput: extraParamState.orderInput,
        },
        queryCondition: extraParamState.queryCondition,
      };
      let res = await userRoleServices.getGovermentClientUsers(params, {
        showLoading: !state.loading,
      });
      return res;
    } catch (error) {}
  },
  {
    defaultExtraParams: {
      orderInput: [{ property: 'id', order: OrderInputType.Desc }],
      queryCondition: '',
    },
    columnsRenderProps: {},
  }
);

function openDialog(row?: API.UserDto) {
  if (row) {
    handleEdit({
      id: row.id,
      userName: row.userName,
      name: row.name,
      phoneNumber: row.phoneNumber,
      password: '',
      remark: row.remark,
    });
  } else {
    handleAdd();
  }
}

const { dialogProps, handleAdd, handleEdit, editForm } = useFormDialog({
  onConfirm: handleAddOrEdit,
  defaultFormParams: {
    id: '',
    userName: '',
    name: '',
    phoneNumber: '',
    password: '',
    remark: '',
  },
});

async function handleAddOrEdit() {
  try {
    let isEdit = !!editForm.id;
    let params: API.CreateAccountInput = {
      name: editForm.name,
      userName: editForm.userName,
      remark: editForm.remark,
      phoneNumber: editForm.phoneNumber,
    };
    let res;
    if (isEdit) {
      (params as API.UpdateGovermentClientUserInput).id = editForm.id;
      res = await userRoleServices.updateGovermentClientUser(params);
    } else {
      (params as API.CreateGoverUserInput).password = editForm.password;
      res = await userRoleServices.createGovermentClientUser(params);
    }
    if (res) {
      Message.successMessage('操作成功');
      getList(isEdit ? paginationState.pageIndex : 1);
    }
  } catch (error) {}
}

const {
  dialogProps: resetPasswordDialogProps,
  handleAdd: handleResetPasswordAdd,
  editForm: resetPasswordEditForm,
} = useFormDialog({
  onConfirm: resetPassword,
  defaultFormParams: {
    userId: '',
    password: '',
  },
});

function openResetPasswordDialog(row: API.UserDto) {
  handleResetPasswordAdd({
    userId: row.id,
    password: '',
  });
}

async function resetPassword() {
  try {
    let params: API.ResetPasswordBaseInput = {
      userId: resetPasswordEditForm.userId,
      password: resetPasswordEditForm.password,
    };
    let res = await accountServices.resetPassword(params);
    if (res) {
      Message.successMessage('操作成功');
      getList(paginationState.pageIndex);
    }
  } catch (error) {}
}
</script>
